;;; -*- lexical-binding: t -*-

(defconst contrast-default-cutoff 0.325)

(defun contrast-compare (cutoff)
  "Compare black and white text on various colours."
  (interactive
   (list (string-to-number
          (read-from-minibuffer "Cutoff value: "
                                (number-to-string contrast-default-cutoff)))))

  (with-current-buffer (get-buffer-create "*Contrast comparison*")
    (erase-buffer)
    (let ((cols nil)
          (seen (make-hash-table :test 'equal)))
      (dolist (name (defined-colors))
        (let ((cv (color-values name)))
          (unless (gethash cv seen)
            (puthash cv cv seen)
            (push (cons cv name) cols))))
      (insert (format "cutoff = %f\n" cutoff))
      (dolist (c (reverse cols))
        (let* ((cv (car c))
               (name (cdr c))
               (hex (apply #'format "#%02x%02x%02x"
                           (mapcar (lambda (x) (/ x 256)) cv)))
               (rgb (mapcar (lambda (x) (/ x 65535.0)) cv))
               (r (expt (nth 0 rgb) 2.2))
               (g (expt (nth 1 rgb) 2.2))
               (b (expt (nth 2 rgb) 2.2))
               (y (+ (* r 0.2126) (* g 0.7152) (* b 0.0722)))
               (fg (if (< y cutoff) "#ffffff" "#000000")))
          (insert
           (propertize
            (format "  %-20.20s  " name)
            'face (list :background name :foreground fg))
           (propertize
            (format " %s " hex)
            'face (list :background name :foreground "#ffffff"))
           (propertize
            (format " %s " hex)
            'face (list :background name :foreground "#000000"))
           (format "  %.5f\n" y)))))
    (let ((pop-up-windows t))
      (display-buffer (current-buffer)))))
